﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Interactivity;
using Infragistics.Windows.DockManager;

namespace XamDockManagerLayout
{
    public class RestoreDockManagerLayout : Behavior<XamDockManager>
    {
        public string Layout
        {
            get { return (string)GetValue(LayoutProperty); }
            set { SetValue(LayoutProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Layout.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty LayoutProperty =
            DependencyProperty.Register("Layout", typeof(string), typeof(RestoreDockManagerLayout), new PropertyMetadata(string.Empty, OnLayoutChanged));

        private static void OnLayoutChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            // make sure this code only executes when the view model is trying to restore the layout
            RestoreDockManagerLayout behavior = d as RestoreDockManagerLayout;
            if (!behavior._saving)
            {
                string layout = (string)e.NewValue;
                if (behavior.AssociatedObject != null && !string.IsNullOrEmpty(layout))
                    behavior.AssociatedObject.LoadLayout(layout);
            }
        }

        protected override void OnAttached()
        {
            base.OnAttached();

            AssociatedObject.Loaded += AssociatedObject_Loaded;
            AssociatedObject.PaneDragEnded += AssociatedObject_PaneDragEnded;
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();

            AssociatedObject.Loaded -= AssociatedObject_Loaded;
            AssociatedObject.PaneDragEnded -= AssociatedObject_PaneDragEnded;
        }

        void AssociatedObject_Loaded(object sender, RoutedEventArgs e)
        {
            // when the dockmanager is restored, the layout needs to be restored as well
            if (!string.IsNullOrEmpty(Layout))
                AssociatedObject.LoadLayout(Layout);
        }

        bool _saving = false;
        void AssociatedObject_PaneDragEnded(object sender, Infragistics.Windows.DockManager.Events.PaneDragEndedEventArgs e)
        {
            // this saves the layout to the viewmodel so it can be restored later
            _saving = true;
            Layout = AssociatedObject.SaveLayout();
            _saving = false;
        }
    }
}
